package jp.agentec.sinaburocast.common.util;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.log4j.Logger;
import org.seasar.framework.util.StringUtil;

/**
 * 外部プロパティファイルを扱うクラス<br>
 * 複数のプロパティファイルやINIファイル等を扱うことが可能。
 *
 * @author tsukada
 *
 */
public class PropertyUtil {
	private static Map<String, Properties> propMap = new ConcurrentHashMap<String, Properties>();
    private static final Logger LOGGER = Logger.getLogger(PropertyUtil.class);

    private static final String baseKey = "Ao8j15gO";
    private static final String cryptKey = baseKey + System.getProperty("property_encrypt_seed");

    /**
     * プロパティファイルを読み込み、Propertiesオブジェクトとして返却
     *
     * @param file　プロパティファイル（.propertiesは付けない）
     * @param reload true ファイルを再読み込みする。　false　キャッシュしてある内容を返す
     * @return　Propertiesオブジェクト
     */
	public static Properties getProperties(String file, boolean reload) {
		Properties properties = propMap.get(file);
		if (properties == null || reload) {
			properties = loadProperies(file, ".properties");
		}
		return properties;
	}

	/**
	 * sinaburocast.propertiesの値を返す
	 * _df付プロパティがあるとき、そちらを優先する。
	 *
	 * @param key　キー値
	 * @return 値　（キーが存在しない場合空文字を返却）
	 */
	public static String getProperty(String key) {
		String val = getProperty("sinaburocast_df", key, false);
		if (StringUtils.isEmpty(val)) {
			val = getProperty("sinaburocast", key, false);
		}
		return val;
	}

	/**
	 * sinaburocast.propertiesの値を返す
	 * contractGroupIdがあるとき、そちらを優先する。
	 *
	 * @param key　キー値
	 * @param contractGroupId　サービス事業者グループＩＤ
	 * @return 値（キーが存在しない場合空文字を返却）
	 */
	public static String getProperty(String key, Integer contractGroupId) {
		String val = "";
		if (contractGroupId != null) {
			val = getProperty(key + "_" + contractGroupId);
		}
		if (StringUtils.isEmpty(val)) {
			val = getProperty(key);
		}
		return val;
	}

	/**
	 * プロパティを設定する（ただしファイルへは反映しない）
	 *
	 * @param file プロパティファイル名（.propertiesを除く）
	 * @param key　キー名
	 * @param val　値
	 */
	public static void setProperty(String file, String key, String val) {
		Properties properties = propMap.get(file);
		if (properties == null) {
			properties = loadProperies(file, ".properties");
		}
		properties.put(key, val);
	}

	/**
	 * プロパティを再読み込みする。
	 *
	 * @param file　プロパティファイル名（.propertiesを除く）
	 */
	public static void reloadProperty(String file) {
		loadProperies(file, ".properties");
	}

	/**
	 * プロパティを取得する
	 *
	 * @param file　プロパティファイル名（.propertiesを除く）
	 * @param key　キー値
	 * @param reload　 true ファイルを再読み込みする。　false　キャッシュしてある内容を返す
	 * @return
	 */
	public static String getProperty(String file, String key, boolean reload) {
		Properties properties = propMap.get(file);
		if (properties == null || reload) {
			properties = loadProperies(file, ".properties");
		}
		if (properties == null) {
			return "";
		}
		return SinaburoUtil.nvl(properties.get(key));
	}

	private static Properties loadProperies(String file, String ext) {
		InputStream is = null;
		try {
			ClassLoader loader = Thread.currentThread().getContextClassLoader();
			URL url = loader.getResource(file + ext);
		    if (url != null) {
	            URLConnection connection = url.openConnection();
	            if (connection != null) {
	                connection.setUseCaches(false);
	                is = connection.getInputStream();
	            }
	        }
		    else {
		    	LOGGER.warn("Resource was not found. [" + file + ext + "]");
		    	is = PropertyUtil.class.getResourceAsStream("/" + file + ext);
		    }
		} catch (IOException e) {
			LOGGER.fatal("load " + file + ext + " failed.", e);
		}
		if (is == null) {
			return null;
		}
		return loadProperties(file, is);
	}

	private static Properties loadProperties(String file, InputStream is) {
		Properties properties = new Properties();
		try {
			if (is == null) {
				LOGGER.fatal("load properties failed: " + file + ".properties may not exist.");
			}
			else {
				properties.load(is);
				is.close();
			}
			propMap.put(file, properties);
		} catch (IOException e) {
			LOGGER.fatal("load properties failed: " + file, e);
			try {
				is.close();
			} catch (IOException e1) {
				LOGGER.error("", e1);
			}
		}
		return properties;
	}

	/**
	 * sinaburocast.propertiesから値を取得して返す。<br>
	 *
	 * @param key　キー名
	 * @return 値　（キーが存在しない場合空文字を返却）
	 */
	public static String getString(String key) {
		return getProperty(key);
	}

	/**
	 * sinaburocast.propertiesから値を取得し、boolean値を返す。<br>
	 *
	 * @param key　キー名
	 * @return　true：値が"true"の場合 false：それ以外の場合
	 */
	public static boolean getBoolean(String key) {
		String str = getProperty(key);
		return (str != null) && str.equals("true");
	}

	/**
	 * sinaburocast.propertiesから値を取得し、short値を返す。<br>
	 *
	 * @param key　キー名
	 * @return　short値（shortではない場合0を返却）
	 */
	public static short getShort(String key) {
		String str = getProperty(key);
		return SinaburoUtil.toShort(str);
	}

	/**
	 * sinaburocast.propertiesから値を取得し、int値を返す。<br>
	 *
	 * @param key　キー名
	 * @return　int値（intではない場合0を返却）
	 */
	public static int getInt(String key) {
		String str = getProperty(key);
		return NumberUtils.toInt(str);
	}

	/**
	 * sinaburocast.propertiesから値を取得し、long値を返す。<br>
	 *
	 * @param key　キー名
	 * @return　long値（longではない場合0Lを返却）
	 */
	public static long getLong(String key) {
		String str = getProperty(key);
		return NumberUtils.toLong(str);
	}

	/**
	 * sinaburocast.propertiesから値を取得し、double値を返す。<br>
	 *
	 * @param key
	 * @return
	 */
	public static double getDouble(String key) {
		String str = getProperty(key);
		return NumberUtils.toDouble(str);
	}

	/**
	 * sinaburocast.propertiesから値を取得し、カンマ区切りで分解し、リストとして返却する。<br>
	 *
	 * @param key　キー名
	 * @return String List（キーが存在しない場合、空リストを返却）
	 */
	public static List<String> getList(String key) {
		return getList(key, false);
	}

	public static List<String> getList(String key, boolean reload) {
		List<String> list = getList("sinaburocast_df", key, reload);
		if (CollectionUtils.isEmpty(list)) {
			list = getList("sinaburocast", key, reload);
		}
		return list;
	}

	/**
	 * sinaburocast.propertiesから値を取得し、カンマ区切りで分解し、リストとして返却する。<br>
	 *
	 * @param key　キー名
	 * @param reload　リロードするか否か
	 * @return String List（キーが存在しない場合、空リストを返却）
	 */
	public static List<String> getList(String file, String key, boolean reload) {
		String str = getProperty(file, key, reload);
		String[] vals = StringUtil.split(str, ",");
		ArrayList<String> list = new ArrayList<String>();
		for (String val : vals) {
			list.add(val.trim());
		}
		return list;
	}

	public static Map<String, Object> getPropertiesStartWith(String key) {
		Map<String, Object> pMap = getPropertiesStartWith("sinaburocast_df", key);
		if (pMap.isEmpty()) {
			pMap = getPropertiesStartWith("sinaburocast", key);
		}
		return pMap;
	}

	public static Map<String, Object> getPropertiesStartWith(String file, String key) {
		Properties properties = propMap.get(file);
		if (properties == null) {
			properties = loadProperies(file, ".properties");
		}

		Map<String, Object> pMap = new HashMap<String, Object>();
		for (Object pKey : properties.keySet()) {
			String pKeyStr = (String)pKey;
			if (pKeyStr.startsWith(key)) {
				pMap.put(pKeyStr, properties.getProperty(pKeyStr));
			}
		}
		return pMap;
	}

}
